<?php

namespace Drupal\content_feedback\Controller;

use Drupal\content_feedback\Form\ContentFeedbackClass;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Render\RendererInterface;
use Drupal\Component\Utility\Html;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for content_feedback routes.
 */
class AdminFeedbackController extends ControllerBase {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs a new AdminFeedbackController object.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(RendererInterface $renderer) {
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer')
    );
  }

  /**
   * Content Feedbacks Admin Page.
   *
   * @param string $type
   *   The type of the overview form ('open' or 'resolved').
   *
   * @return array
   *   Returns renderable array that display all content feedbacks.
   */
  public function content($type = 'open') {
    $header = [
      ['data' => $this->t('Date'), 'field' => 'updated', 'sort' => 'desc'],
      ['data' => $this->t('User'), 'field' => 'name', 'sort' => 'asc'],
      'email' => $this->t('Email'),
      'path' => $this->t('Location'),
      'ipaddress' => $this->t('IP Address'),
      'message' => $this->t('Feedback'),
      'operations' => $this->t('Actions'),
    ];

    $rows = [];
    $feedbacksList = ContentFeedbackClass::getAllContentFeedbacks($header, $type);

    if ($feedbacksList) {
      foreach ($feedbacksList as $content) {
        $rows[] = $this->buildFeedbackRow($content);
      }

      $build['heading'] = [
        '#type' => 'html_tag',
        '#tag' => 'h3',
        '#value' => $type === 'open' ? $this->t('Open Content Feedback') : $this->t('Resolved Content Feedback'),
      ];

      $build['config_table'] = [
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#attributes' => ['class' => ['content-feedback-table']],
      ];

      $build['pager'] = ['#type' => 'pager'];
    }
    else {
      $build = [
        '#type' => 'markup',
        '#markup' => '<p>' . $this->t('No Content Feedback Found.') . '</p>',
      ];
    }

    return $build;
  }

  /**
   * Builds a single feedback row.
   *
   * @param object $content
   *   The feedback content object.
   *
   * @return array
   *   A renderable array representing the feedback row.
   */
  private function buildFeedbackRow($content) {
    $actionLink = [
      Link::createFromRoute($this->t('Edit'), 'content_feedback_edit.form', ['id' => $content->id])->toRenderable(),
      Link::createFromRoute($this->t('Delete'), 'content_feedback_delete.form', ['id' => $content->id])->toRenderable(),
    ];

    // Render the action links using the renderer service.
    $actionLinks = $this->renderer->render($actionLink);
    $submitDate = DrupalDateTime::createFromTimestamp($content->updated, new \DateTimeZone(date_default_timezone_get()));

    return [
      'data' => [
        $submitDate->format('m/d/Y - H:i'),
        Html::escape($content->name),
        Html::escape($content->email),
        Html::escape($content->path),
        Html::escape($content->ipaddress),
        Html::escape($content->message),
        $actionLinks,
      ],
    ];
  }

}
