<?php

namespace Drupal\content_feedback\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Utility\Html;

/**
 * Defines the AddContentFeedback form.
 */
class AddContentFeedback extends FormBase {

  /**
   * The content feedback config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * The rendering service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Constructs AddContentFeedback object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer interface.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, RendererInterface $renderer, RequestStack $request_stack) {
    $this->config = $config_factory->get('content_feedback.settings');
    $this->renderer = $renderer;
    $this->requestStack = $request_stack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('renderer'),
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_feedback_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config;
    $path = $this->requestStack->getCurrentRequest()->server->get('HTTP_REFERER') ?: '/';

    $form['#prefix'] = '<div id="modal_form">';
    $form['#suffix'] = '</div>';

    $form['status_messages'] = [
      '#type' => 'status_messages',
      '#weight' => -10,
    ];

    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Name'),
      '#default_value' => $this->currentUser()->getAccountName(),
      '#required' => $config->get('name.required') === 'required',
    ];

    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Email'),
      '#default_value' => $this->currentUser()->getEmail(),
      '#required' => $config->get('email.required') === 'required',
    ];

    $form['message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Do you have any suggestions to make this page better?'),
      '#rows' => 3,
      '#required' => TRUE,
    ];

    $form['path'] = [
      '#type' => 'hidden',
      '#value' => $path,
    ];

    $form['ipaddress'] = [
      '#type' => 'hidden',
      '#value' => $this->requestStack->getCurrentRequest()->getClientIp(),
    ];

    $form['actions']['send'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
      '#ajax' => [
        'callback' => [$this, 'submitContentFeedback'],
        'wrapper' => 'modal_form',
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {}

  /**
   * AJAX callback handler for content feedback submission.
   */
  public function submitContentFeedback(array $form, FormStateInterface $form_state) {
    $response = new AjaxResponse();

    if ($form_state->hasAnyErrors()) {
      $response->addCommand(new ReplaceCommand('#modal_form', $form));
    }
    else {
      $values = $form_state->getValues();
      ContentFeedbackClass::addContentFeedback(
        Html::escape($values['name']),
        Html::escape($values['email']),
        Html::escape($values['message']),
        Html::escape($values['path']),
        Html::escape($values['ipaddress']),
        time()
      );

      $element = [
        '#type' => 'markup',
        '#markup' => $this->t('Your content feedback has been submitted successfully.'),
        '#prefix' => '<div id="success-message">',
        '#suffix' => '</div>',
      ];
      $response->addCommand(new ReplaceCommand('#modal_form', $this->renderer->render($element)));
    }

    return $response;
  }

}
